using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Hazmhox
{
    public class Avakin : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Hazmhox"; // edited by VamTastic (original by Frief )
        public string pluginName = "Lab Doors";
        public string pluginVersion = "1";
        public string pluginDate = "06/06" + "/2022";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    



        public GameObject player;
        public GameObject KS_flynns_hideout;

        protected JSONStorableFloat ScrubA;
        protected JSONStorableFloat ScrubB;
		
	    protected JSONStorableFloat yRotation;
        protected JSONStorableFloat yRotation2;		


        private float previousValue1;
        private float previousValue2;		

        private float previousZValue;
        private float currentZValue;

	    public Transform ObjectRoot;
        public Transform door1;
        public Transform door2;
		
        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        public override void Init()
        {
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
				{
	                SuperController.LogError("flynns_hideout - Please add this script on a Custom Unity Asset ( script disabled).");
  	                return;
  		        }


                //**************  ScrubA
                ScrubA = new JSONStorableFloat("Door 1", 0f, OnSliderChanged, 0f, 1f);
                RegisterFloat(ScrubA);
                CreateSlider(ScrubA, false);
                          

                //**************  ScrubB
                ScrubB = new JSONStorableFloat("Door 2", 0f, OnSliderChanged2, 0f, 1f);				
                RegisterFloat(ScrubB);
                CreateSlider(ScrubB, false);
				
								

              // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Credits to Hazmhox.\n\n" +
					"Flynns Lab Doors.\n\n" +
                    "The doors can be triggered using Acidbubbles Timeline to open and close.\n\n" +
                    "if you have a door open and save the scene the position of a door is also saved.\n\n" +
                    "</color>" );
               UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
			// I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
			// In reality, that's even cleaner to do it that way
			if( initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading ) {
                initLoops++;
                InitEditor();
            }
			
			if( initLoops == maxInitLoops ) {
                SuperController.LogError("flynns_hideout : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }
        }

        private bool InitEditor() {
            if( editorInitialized == true ) return true;
        
            ObjectRoot = getObjectRoot( containingAtom.reParentObject, "flynns_hideout" );
            if( ObjectRoot == null ) {
                return false;
            }
			
            door1 = getChildRoot(ObjectRoot, "door1");
            door2 = getChildRoot(ObjectRoot, "door2");
            // I found my transforms, now the rest
            if( door1 && door2 ) {
				editorInitialized = true;
				
				// Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
				// I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
				// the float val param of the callbacks is useless
				OnSliderChanged( 0f );
				OnSliderChanged2( 0f );			
				return true;
            } else {
                SuperController.LogError("flynns_hideout : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }
        
            return false;
        }

		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		// This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
		private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				SuperController.LogMessage(">>" + child.name);
				if( child.name == searchName ) {
					
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

		//  Door 1
        void OnSliderChanged( float rotation )
        {
            
            {
				if( door1 == null ) return;
				
                var Slider1 = ScrubA.val;
                var a = Slider1 - previousValue1;
				door1.transform.Rotate(Vector3.forward * a * -90);
				previousValue1 = ScrubA.val;
            }

        }

		// Door 2
        void OnSliderChanged2( float rotation )
        {

            {
				if( door2 == null ) return;
				
                var Slider2 = ScrubB.val;
                var b = Slider2 - previousValue2;
                door2.transform.Rotate(Vector3.forward * b * 90);
                previousValue2 = ScrubB.val;
            }

        }	

    }
	
}